# Zeppelin „Gaffelin“

## Übersicht
1. [Projektbeschreibung](#projektbeschreibung)
2. [Benötigte Software und Bibliotheken](#benötigte-software-und-bibliotheken)
3. [Code herunterladen und verwenden](#code-herunterladen-und-verwenden)
4. [Hinweise zur Benutzung](#hinweise-zur-benutzung)

## Projektbeschreibung
Im Rahmen des _MINTgrün-Robotik-Labors_ an der [TU Berlin](https://www.tu.berlin/) hat sich unsere Projektgruppe dazu entschieden einen Zeppelin(roboter) zu bauen. Die **Projektdokumentation** findet sich [hier](https://www.mintgruen.tu-berlin.de/robotikWiki/doku.php?id=projektewise21:fliegenderroboter:doku).
Das direkte Ansteuern der Aktuatoren und Sensoren auf dem Zeppelin erfolgt über einen Arduino-Microcontroller. Dieser sendet die Sensorwerte über Bluetooth an einen bluetoothfähigen Computer und erhält von diesem die Steuerungsbefehle für die Aktuatoren. Auf dem Computer läuft eine Processing-Anwendung, welche die Sensorwerte verarbeitet und Steuerungsbefehle berechnet. Hierbei kann zwischen zwei Modi unterschieden werden:
1. Manueller Modus (_ManualMode_)
2. Licht-Folge-Modus (_LightFollowMode_).

Im _ManualMode_ erfolgt die Steuerung des Zeppelins manuell über die Computertastatur (<kbd>w</kbd> Schub, <kbd>a</kbd> links, <kbd>d</kbd> rechts, <kbd>↑</kbd> hoch, <kbd>↓</kbd> runter) oder über einen Game-Controller.

Im _LightFollowMode_ erfolgt die Steuerung durch die Auswertung der Sensordaten. Der Zeppelin soll hierbei automatisch einer Lichtquelle folgen und die Kollission mit Hindernissen vermeiden.

In beiden Modi werden die eintreffenden Sensorwerte und der Akkustand auf dem Computerbildschirm visualisiert.

## Benötigte Software und Bibliotheken
- [Arduino IDE](https://www.arduino.cc/en/software)
- [Processing IDE](https://processing.org/download)

### Installation von zusätzlichen Processing Bibliotheken
1. _Processing IDE_ öffnen

#### Game Control Plus
2. `Sketch` >> `Library importieren ...` >> `Library hinzufügen ...`
3. `Filter` >> _Game Control Plus_ eingeben
4. `Game Control Plus` auswählen
5. installieren

#### grafica
2. `Sketch` >> `Library importieren ...` >> `Library hinzufügen ...`
3. `Filter` >> _grafica_ eingeben
4. `grafica` auswählen
5. installieren

## Code herunterladen und verwenden
```shell
$ git clone https://git.tu-berlin.de/fliegenderroboter/Zeppelin.git
```

### Arduino
1. in Ordner `Zeppelin_Arduino-Code` wechseln
2. `Zeppelin_Arduino-Code.ino` mit _Arduino IDE_ öffnen
3. Microcontroller _Arduino Nano_ mit Computer verbinden
4. `Sketch` >> `Hochladen`

### Processing
1. in Ordner `Zeppelin_Processing_Code` wechseln
2. `Zeppelin_Processing_Code.pde` mit _Processsing IDE_ öffnen
3. Zeppelin anschalten
4. `Sketch` >> `Starten`

## Hinweise zur Benutzung
Der gesamte Source Code ist an unsere eigenen Voraussetzungen angepasst und muss zur eigenen Benutzung wahrscheinlich abgeändert werden.

1. **Pinbelegung überprüfen** und gegebenenfalls im Arduino-Code oder am Arduino ändern. Die Code-Zeilen `44-55` in `Zeppelin_Arduino-Code/Zeppelin_Arduino-Code.ino` sind hierfür relevant:
```c
const byte ULTRA_SONIC_RIGHT_PIN = 2;
const byte ULTRA_SONIC_LEFT_PIN = 3;

//const byte ARDUINO_RX_PIN = 5; // für SoftwareSerial Kommentierung aufheben
//const byte ARDUINO_TX_PIN = 6; // für SoftwareSerial Kommentierung aufheben
const byte SERVO_FIN_UP_DOWN_PIN = 9;
const byte SERVO_FIN_LEFT_RIGHT_PIN = 10;
const byte SERVO_ESC_PIN = 11;
const byte BATTERY_VOLTAGE_PIN = A0;
const byte PT_RIGHT_PIN = A5;
const byte PT_LEFT_PIN = A6;
const byte PT_DOWN_PIN = A7;
```

2. **Bluetooth-Port überprüfen** und gegebenfalls für den entsprechenden Computer anpassen. Gerade zwischen Linux und Windows unterscheiden sich die Portnamen. Wenn der entsprechenden Portname bekannt ist, kann er in `Zeppelin_Processing_Code/Zeppelin_Processing_Code.pde` in folgender Zeile (`63`) zwischen den Anführungszeichen angepasst werden:
```java
String portnameBluetooth = "/dev/rfcomm0";
```

3. Ist **kein Game-Controller** ansgeschlossen? In diesem Fall müssen im Processing-Code folgenden Stellen durch zwei vorangestellte Schrägstriche auf folgende Art auskommentiert werden:

`Zeppelin_Processing_Code/Zeppelin_Processing_Code.pde` Zeile `79`:
```java
// GPC.setup();
```

`Zeppelin_Processing_Code/ManualMode.pde` Zeile `32`:
```java
// GPC.loop();
```

4. Wenn ein **Game-Controller angeschlossen** ist, kann es passieren das die einzelnen Steuerelemente des Game-Controllers und der Game-Controller selbst eine andere Bezeichnung als im Processing-Code hat. Um dies zu testen, kann ein mitgeliefertes Beispiel der _Game Control Plus_ Bibliothek verwendet werden. Dazu:

    1. _Processing IDE_ öffnen
    2. `Datei` >> `Beispiele ...`
    3. `Contributed Libraries` >> `Game Control Plus` >> `Gcp_gamepad`
    4. `Sketch` >> `Starten`
    
    Nun kann der entsprechende Game-Controller ausgewählt werden und die verschiedenen Steuerelemente mit ihren Bezeichnern festgestellt werden.

    Als nächstes können die Bezeichner zwischen den Anführungszeichen in `Zeppelin_Processing_Code/GamePadControl.pde` Zeile `66-76` angepasst werden:
```java
modeButton = device.getButton("Mode");
AButton = device.getButton("A");
BButton = device.getButton("B");
startButton = device.getButton("Unknown");
    
sliderLeftX = device.getSlider("x");    // Steuerknüppel links x
sliderLeftY = device.getSlider("y");    // Steuerknüppel links y
sliderLeftZ = device.getSlider("z");    // LT (hinten)
sliderRightX = device.getSlider("rx");  // Steuerknüppel rechts x
sliderRightY = device.getSlider("ry");  // Steuerknüppel rechts y
sliderRightZ = device.getSlider("rz");  // RT (hinten)
```

5. **Benutzung des Game-Controllers**: Nachdem der Processing-Sketch gestartet wurde, lässt sich die Steuerung mit Game-Controller über den als `modeButton` festgelegten Knopf aktivieren und deaktivieren. Die als `AButton` und `BButton` festgelegten Knöpfe ermöglichen es den Game-Controller in zwei unterschiedlichen Mapping-Modi zu verwenden. Genaueres dazu in der Kommentierung im Source-Code in `Zeppelin_Processing_Code/GamePadControl.pde`.
