public class Battery {
  private final String caption;
  private final float xMin;
  private final float xMax;
  private final float yMin;
  private final float yMax;
  private float charge;
  
  Battery(float xMin, float xMax, float yMin, float yMax, String caption) {
    this.caption = caption;
    this.xMin = xMin;  // kleinste x Wert | kleinste x Koordinate
    this.xMax = xMax;  // größte x Wert | größte x Koordinate
    this.yMin = yMin;  // größte y Wert | kleinste y Koordinate
    this.yMax = yMax;  // kleinste y Wert | größte y Koordinate
  }
  
  // gibt für die Akkuspannung eines 3.7V Akkus den passenden Batteriestand zurück
  private float voltageToBatteryLevel(float voltage) {
    if (voltage >= 4.2) {
      return 1.0;
    } else if (voltage >= 4.1) {
      return 0.98;
    } else if (voltage >= 4.0) {
      return 0.95;
    } else if (voltage >= 3.9) {
      return 0.90;
    } else if (voltage >= 3.8) {
      return 0.75;
    } else if (voltage >= 3.7) {
      return 0.55;
    } else if (voltage >= 3.6) {
      return 0.30;
    } else if (voltage >= 3.5) {
      return 0.20;
    } else if (voltage >= 3.4) {
      return 0.08;
    } else if (voltage >= 3.3) {
      return 0.02;
    } else if (voltage < 3.3){
      return 0.00;
    } else {
      return 0;
    }
  }
  
  public void drawLines() {  // Umriss zeichnen
    clearAll();
    stroke(0);
    strokeWeight(2);
    noFill();
    int extra = 2;
    rect(xMin-extra, yMin-extra, xMax-xMin+extra, yMax-yMin+extra);
    
    fill(0);
    textAlign(CENTER);
    text(caption, xMin+ (xMax-xMin)/2, yMax+15);
  }
  
  public void drawCharge(float voltage) {
    clearCharge();
    noStroke();
    float newCharge = voltageToBatteryLevel(voltage) * (yMax-yMin);
    if(voltageToBatteryLevel(voltage) > 0.7) {
      fill(0, 255, 0);  //grün
    }
    else if(voltageToBatteryLevel(voltage) < 0.3) {
      fill(255, 0, 0);  //rot
    }
    else {
      fill(255, 255, 0);  //gelb
    }
    rect(xMin, yMax, xMax-xMin, -newCharge);
    text(str((int)(voltageToBatteryLevel(voltage)*100)) + "%", xMin+ (xMax-xMin)/2, yMax+30);
  }
  
  private void clearCharge() {
    noStroke();
    fill(255);
    rect(xMin, yMax, xMax-xMin, yMin-yMax);
    rect(xMin-10, yMax+20, xMax-xMin+20, 30);
  }
  
  private void clearAll() {
    noStroke();
    fill(255);
    int extra = 2;
    rect(xMin-extra, yMin-extra, xMax-xMin+extra, yMax-yMin+20);
  }
};
