// Klasse beihaltet die Funktionen des Licht-folgen-Modus
public class LightFollowMode {
  private VirtualArduino vArduino;
  private ServoFin finUpDown;
  private ServoFin finLeftRight;
  private ServoFin motorESC;
  private Crossdiagram servoDiagram;
  private Crossdiagram escDiagram;
  private long lastCall = millis();     // speichert den Zeitpunkt des letzten Aufrufs
  private int epsilon = 30;
  private float criticalDistance = 100; // die kritische Distanz von Hindernissen wird auf 100cm festgelegt
  
  LightFollowMode(VirtualArduino vArduino, ServoFin finUpDown, ServoFin finLeftRight, ServoFin motorESC, Crossdiagram servoDiagram, Crossdiagram escDiagram) {
    // andere Objekte, auf die zugegriffen werden muss:
    this.vArduino = vArduino;
    this.finUpDown = finUpDown;
    this.finLeftRight = finLeftRight;
    this.motorESC = motorESC;
    this.servoDiagram = servoDiagram;
    this.escDiagram = escDiagram;
  }
  
  // Kollisionskursdetektion:
  // gibt true zurück, wenn sich ein Objekt die kritischen Distanz unterschreitet
  // gibt false zurück, wenn das nicht der Fall ist
  boolean collisionCourseDetected() {
    float distanceLeft = vArduino.getDistanceLeftValue();                      // Abstand zu Objekten links vor'm Arduino
    float distanceRight = vArduino.getDistanceRightValue();                    // Abstand zu Objekten rechts vor'm Arduino
    if (distanceLeft < criticalDistance || distanceRight < criticalDistance) {
      return true;
    } else {
      return false;
    }
  }
  
  // Ausweichmanöver einleiten:
  // wenn sich das Objekt auf der linken Seite befindet, weicht der Zeppelin nach rechts aus
  // wenn sich das Objekt auf der rechten Seite befindet, weicht der Zeppelin nach links aus
  // wenn sich das Objekt in der Mitte befindet, hält der Zeppelin an.
  void evasionManeuver() {
    float distanceLeft = vArduino.getDistanceLeftValue();
    float distanceRight = vArduino.getDistanceRightValue();
    if (distanceLeft < criticalDistance && !(distanceRight < criticalDistance)) {
      //move right
      finLeftRight.sendNewPosition(-80);
    } else if (!(distanceLeft < criticalDistance) && distanceRight < criticalDistance) {
      //move left
      finLeftRight.sendNewPosition(80);
    } else if (distanceLeft < criticalDistance && distanceRight < criticalDistance) {
      //engine stop
      motorESC.sendNewPosition(0);
    }
  }
  
  // Funktion, die in der draw-Funktion aufgerufen werden soll
  // sendet die berechneten Werte an den Arduino
  // Wenn Kollisionsgefahr besteht, wird ein Ausweichmanöver eingeleitet,
  //   sonst wird die Richtung anhand der PT-Daten berechnet
  void loop() {
    if (millis() - lastCall > 200) {
      lastCall = millis();
      
      if (collisionCourseDetected()) {
        println("Eisberg voraus!");
        evasionManeuver();
      } else {
        int pt_right = vArduino.getPTRightValue();
        int pt_left = vArduino.getPTLeftValue();
        int pt_down = vArduino.getPTDownValue();
        if(pt_right - pt_down > epsilon || pt_left - pt_down > epsilon) {
          // nach oben bewegen
          if (vController.getUpDownPosition() != 0.7) {
            vController.updateUpDownPosition(0.7);
            finUpDown.sendNewPosition((int) map(vController.getUpDownPosition(), -1, 1, -90, 90));
            servoDiagram.drawFinPosition(map(vController.getUpDownPosition(), -1, 1, 0, 180), map(vController.getLeftRightPosition(), -1, 1, 0, 180));
          }
        }
        else if(pt_down - pt_right > epsilon && pt_down - pt_left > epsilon) {
          // nach unten bewegen
          if (vController.getUpDownPosition() != -0.7) {
            vController.updateUpDownPosition(-0.7);
            finUpDown.sendNewPosition((int) map(vController.getUpDownPosition(), -1, 1, -90, 90));
            servoDiagram.drawFinPosition(map(vController.getUpDownPosition(), -1, 1, 0, 180), map(vController.getLeftRightPosition(), -1, 1, 0, 180));
          }
        }
        else {
          // mittig bewegen (keine Änderung der Flugrichtung entlang der Querachse)
          if (vController.getUpDownPosition() != 0) {
            vController.updateUpDownPosition(0);
            finUpDown.sendNewPosition((int) map(vController.getUpDownPosition(), -1, 1, -90, 90));
            servoDiagram.drawFinPosition(map(vController.getUpDownPosition(), -1, 1, 0, 180), map(vController.getLeftRightPosition(), -1, 1, 0, 180));
          }
        }
        
        if(pt_right - pt_left > epsilon) {
          // nach rechts bewegen
          if (vController.getLeftRightPosition() != 0.5) {
            vController.updateLeftRightPosition(0.5);
            finLeftRight.sendNewPosition((int) map(vController.getLeftRightPosition(), -1, 1, -90, 90));
            servoDiagram.drawFinPosition(map(vController.getUpDownPosition(), -1, 1, 0, 180), map(vController.getLeftRightPosition(), -1, 1, 0, 180));
          }
        }
        else if(pt_left - pt_right > epsilon) {
          // nach links bewegen 
          if (vController.getLeftRightPosition() != -0.5) {
            vController.updateLeftRightPosition(-0.5);
            finLeftRight.sendNewPosition((int) map(vController.getLeftRightPosition(), -1, 1, -90, 90));
            servoDiagram.drawFinPosition(map(vController.getUpDownPosition(), -1, 1, 0, 180), map(vController.getLeftRightPosition(), -1, 1, 0, 180));
          }
        }
        else {
          // nach vorne bewegen (keine Änderung der Flugrichtung entlang der vertikalen Achse)
          if (vController.getLeftRightPosition() != 0) {
            vController.updateLeftRightPosition(0);
            finLeftRight.sendNewPosition((int) map(vController.getLeftRightPosition(), -1, 1, -90, 90));
            servoDiagram.drawFinPosition(map(vController.getUpDownPosition(), -1, 1, 0, 180), map(vController.getLeftRightPosition(), -1, 1, 0, 180));
          }
        }
        // Motorgeschwindigkeit auf 30% stellen:
        if (vController.getMotorDrive() != 0.4) {
          vController.updateMotorDrive(0.4);
          motorESC.sendNewPosition((int) map(vController.getMotorDrive(), 0, 1, 0, 180)); // für Event-basierte Datenübertragung
          escDiagram.drawFinPosition(map(vController.getMotorDrive(), 0, 1, 0, 180), 90);
        }
      }
    }
  }
}
