// Klasse beihaltet die Funktionen des manuellen Modus
public class ManualMode {
  private VirtualController vController;
  private GamePadControl GPC;
  private VirtualArduino vArduino;
  private ServoFin finUpDown;
  private ServoFin finLeftRight;
  private ServoFin motorESC;
  private Crossdiagram servoDiagram;
  private Crossdiagram escDiagram;
  
  private long lastCall = millis();
  
  ManualMode(VirtualController vController, GamePadControl GPC, VirtualArduino vArduino, ServoFin finUpDown, ServoFin finLeftRight, ServoFin motorESC, Crossdiagram servoDiagram, Crossdiagram escDiagram) {
    this.vController = vController;
    this.GPC = GPC;
    this.vArduino = vArduino;
    this.finUpDown = finUpDown;
    this.finLeftRight = finLeftRight;
    this.motorESC = motorESC;
    this.servoDiagram = servoDiagram;
    this.escDiagram = escDiagram;
  }
  
  // Funktion, die in der draw-Funktion aufgerufen werden soll
  void loop() {
    if (millis() - lastCall > 50) {
      lastCall = millis();
      // finUpDown.sendNewPosition((int) map(vController.getUpDownPosition(), -1, 1, -90, 90)); // Änderung wird konstant gesendet
      // finLeftRight.sendNewPosition((int) map(vController.getLeftRightPosition(), -1, 1, -90, 90));
      // motorESC.sendNewPosition((int) map(vController.getMotorDrive(), 0, 1, 0, 180));
      GPC.loop();  // auskommentieren, wenn kein Game-Controller angeschlossen ist
    }
  }
  
  // Funktion, die im Falle eines Tastendrucks aufgerufen werden soll und die Steuerung über wasd + PFEILTASTE_OBEN, PFEILTASTE_UNTEN ermöglichen soll
  // aktualisiert die Zustände des virtuellen Controllers
  // sendet die Controller-Positionen übersetzt an den Arduino
  void keyPressed(char key, int keyCode) {
    switch(key) {
      case 'w':
        if (vController.getMotorDrive() != 0.5) {
          vController.updateMotorDrive(0.5);
          motorESC.sendNewPosition((int) map(vController.getMotorDrive(), 0, 1, 0, 180)); // für Event-basierte Datenübertragung
          escDiagram.drawFinPosition(map(vController.getMotorDrive(), 0, 1, 0, 180), 90);
        }
        break;
      case 'a':
        if (vController.getLeftRightPosition() != -1) {
          vController.updateLeftRightPosition(-1);
          finLeftRight.sendNewPosition((int) map(vController.getLeftRightPosition(), -1, 1, -45, 45));
          servoDiagram.drawFinPosition(map(vController.getUpDownPosition(), -1, 1, 27, 153), map(vController.getLeftRightPosition(), -1, 1, 45, 135));
        }
        break;
      case 's':
        // voraussichtlich kein Rückwärtsgang
        break;
      case 'd':
        if (vController.getLeftRightPosition() != 1) {
          vController.updateLeftRightPosition(1);
          finLeftRight.sendNewPosition((int) map(vController.getLeftRightPosition(), -1, 1, -45, 45));
          servoDiagram.drawFinPosition(map(vController.getUpDownPosition(), -1, 1, 27, 153), map(vController.getLeftRightPosition(), -1, 1, 45, 135));
        }
        break;
    }
    switch(keyCode) {
      case UP:
        if (vController.getUpDownPosition() != 1) {
          vController.updateUpDownPosition(1);
          finUpDown.sendNewPosition((int) map(vController.getUpDownPosition(), -1, 1, -63, 63));
          servoDiagram.drawFinPosition(map(vController.getUpDownPosition(), -1, 1, 27, 153), map(vController.getLeftRightPosition(), -1, 1, 45, 135));
        }
        break;
      case DOWN:
        if (vController.getUpDownPosition() != -1) {
          vController.updateUpDownPosition(-1);
          finUpDown.sendNewPosition((int) map(vController.getUpDownPosition(), -1, 1, -63, 63));
          servoDiagram.drawFinPosition(map(vController.getUpDownPosition(), -1, 1, 27, 153), map(vController.getLeftRightPosition(), -1, 1, 45, 135));
        }
        break;
    }
  }
  
  // Funktion, die aufgerufen werden soll, wenn Taste losgelassen wird
  // aktualisiert die Zustände des virtuellen Controllers
  void keyReleased(char key, int keyCode) {
    switch(key) {
      case 'w':
        vController.updateMotorDrive(0);
        motorESC.sendNewPosition((int) map(vController.getMotorDrive(), 0, 1, 0, 180));
        escDiagram.drawFinPosition(map(vController.getMotorDrive(), 0, 1, 0, 180), 90);
        break;
      case 'a':
        vController.updateLeftRightPosition(0);
        finLeftRight.sendNewPosition((int) map(vController.getLeftRightPosition(), -1, 1, -45, 45));
        servoDiagram.drawFinPosition(map(vController.getUpDownPosition(), -1, 1, 27, 153), map(vController.getLeftRightPosition(), -1, 1, 45, 135));
        break;
      case 's':
        // voraussichtlich kein Rückwärtsgang
        break;
      case 'd':
        vController.updateLeftRightPosition(0);
        finLeftRight.sendNewPosition((int) map(vController.getLeftRightPosition(), -1, 1, -45, 45));
        servoDiagram.drawFinPosition(map(vController.getUpDownPosition(), -1, 1, 27, 153), map(vController.getLeftRightPosition(), -1, 1, 45, 135));
        break;
    }
    switch(keyCode) {
      case UP:
        vController.updateUpDownPosition(0);
        finUpDown.sendNewPosition((int) map(vController.getUpDownPosition(), -1, 1, -63, 63));
        servoDiagram.drawFinPosition(map(vController.getUpDownPosition(), -1, 1, 27, 153), map(vController.getLeftRightPosition(), -1, 1, 45, 135));
        break;
      case DOWN:
        vController.updateUpDownPosition(0);
        finUpDown.sendNewPosition((int) map(vController.getUpDownPosition(), -1, 1, -63, 63));
        servoDiagram.drawFinPosition(map(vController.getUpDownPosition(), -1, 1, 27, 153), map(vController.getLeftRightPosition(), -1, 1, 45, 135));
        break;
    }
  }
}
