// Klasse zur Steurung der der Servos und des Antriebmotors
public class ServoFin {
  private final SerialCommunication seriComObj;
  private final int finKey;                      // Key für die Kommunikation mit Arduino siehe class SerialCommunication
  private final int zeroFinPosition;             // Nullposition des Servos
  private final int minFinPosition;              // Minimalposition des Servos relativ zur Nullposition
  private final int maxFinPosition;              // Maximalposition des Servos relativ zur Nullposition
  private int finPosition = 0;                   // aktuelle Position des Servos relativ zur Nullposition
  
  ServoFin(SerialCommunication seriComObj, int finKey, int zeroFinPosition, int minFinPosition, int maxFinPosition) {
    this.seriComObj = seriComObj;
    this.finKey = finKey;
    this.zeroFinPosition = zeroFinPosition;
    this.minFinPosition = minFinPosition;
    this.maxFinPosition = maxFinPosition;
  }
  
  // sendet die aktuelle Position an den Arduino
  public void setup(int finPosition) {
    setPosition(finPosition);
    sendNewPosition(finPosition);
  }
  
  // gibt die aktuelle Position zurück
  public int getPosition() {
    return this.finPosition;
  }
  
  // setzt die neue Position
  public boolean setPosition(int newFinPosition) {
    if (newFinPosition <= maxFinPosition && newFinPosition >= minFinPosition) {
      this.finPosition = newFinPosition;
      return true;
    } else {
      return false;
    }
  }
  
  // sendet die Position der Finne (Winkel) auf einen neuen Wert
  public boolean sendNewPosition(int newFinPosition) {
    if (setPosition(finPosition)) {
      boolean status = seriComObj.sendData(finKey, str(newFinPosition + zeroFinPosition));
      //print("neue Servo-Position an Arduino für ");
      //print(finKey);
      //print(" gesendet: ");
      //println(newFinPosition);
      return status;
    } else {
      return false;
    }
  }
  
  // ändert die Position um einen bestimmten Wert
  public boolean changePosition(int difference) {
    return sendNewPosition(finPosition + difference);
  }
}
