import grafica.*;                   // Library zum Plotten der Daten

import net.java.games.input.*;      // für GamePadAPI
import org.gamecontrolplus.*;       // für GamePadAPI
import org.gamecontrolplus.gui.*;   // für GamePadAPI

import processing.serial.*;

// Modi:
final int CALLIBRATION = 0; // existiert bisher noch nicht
final int MANUAL = 1;       // manuelle Steuerung über Computer-Tastatur
final int LIGHT = 2;        // Licht-folge-Modus: autonome Steuerung durch Computer zum Folgen einer Lichtquelle

int currentMode;            // speichert den aktuellen Modus

ManualMode manualMode;      // neues Objekt mit den für die manuelle Steuerung notwendigen Methoden 
LightFollowMode lightMode;  // neues Objekt mit den für den Licht-folge-Modus notwendigen Methoden

Serial serialObject;
SerialCommunication seriCom; // Objekt für die weitere Kommunikation mit dem Arduino

VirtualController vController;      // neuer virtueller Controller, der die aktuellen Zustände der Steuerungsbefehle speichern soll
VirtualArduino vArduino;            // neuer virtueller Arduino, der die aktuellen Zustände, Messwerte und Servo-Positionen des Arduinos speichern soll

Crossdiagram servoDiagramVArduino;          // neues Kreuzdiagramm, welches die aktuellen Servopositionen des Arduinos visualisieren soll
Crossdiagram escDiagramVArduino;            // neues Kreuzdiagramm, welches den aktuellen Wert des Antriebsmotors am Arduino visualisieren soll
Crossdiagram servoDiagramVController;       // neues Kreuzdiagramm, welches die aktuellen Servopositionen des virtuellen Controllers visualisieren soll
Crossdiagram escDiagramVController;         // neues Kreuzdiagramm, welches den aktuellen Wert des Antriebsmotors vom virtellen Controller visualisieren soll
LivePlotDiagram PTDiagramVArduino;          // neuer Echtzeitplot der Phototranistordaten
LivePlotDiagram distanceDiagramVArduino;  // neuer Echtzeitplot der Ultraschalldistanzmessungen
Battery batteryStatus;


ServoFin finUpDown;     // Objekt zur Steuerung der Up-Down-Steuerflosse
ServoFin finLeftRight;  // Objekt zur Steuerung der Left-Right-Steuerflosse
ServoFin motorESC;      // Objekt zur Steuerung des Antriebsmotors


void setup() {
  size(800, 550);
  background(255);
  
  // Setup der Diagramme:
  servoDiagramVArduino = new Crossdiagram(20, 120, 20, 120, "Servo V-Arduino", "-90°", "0°", "90°"); // float xMin, float xMax, float yMin, float yMax, String caption, String minText, String middleText, String maxText
  escDiagramVArduino = new Crossdiagram(150, 250, 20, 120, "ESC V-Arduino", "0", "90", "180");
  servoDiagramVController = new Crossdiagram(20, 120, 150, 250, "Servo V-Controller", "-90°", "0°", "90°");
  escDiagramVController = new Crossdiagram(150, 250, 150, 250, "ESC V-Controller", "0", "90", "180");
  
  PTDiagramVArduino = new LivePlotDiagram(this, 300, 0, 450, 300, "PT-Diagramm", "t[s]", "Lichtintensität[arb. unit]"); // PApplet parent, float xPos, float yPos, float plotWidth, float plotHeight, String caption, String xAxisLabel, String yAxisLabel
  PTDiagramVArduino.addLayer("PT rechts", color(255, 0, 0));
  PTDiagramVArduino.addLayer("PT links", color(0, 255, 0));
  PTDiagramVArduino.addLayer("PT unten", color(0, 0, 255));
  
  distanceDiagramVArduino = new LivePlotDiagram(this, 0, 300, 750, 250, "Distanz", "t[s]", "s[cm]");
  distanceDiagramVArduino.addLayer("Distanz links", color(0, 0, 255));
  distanceDiagramVArduino.addLayer("Distanz rechts", color(255, 0, 0));
  
  batteryStatus = new Battery(width - 30, width - 10, 20, 60, "Akku");  // float xMin, float xMax, float yMin, float yMax, String caption
  batteryStatus.drawLines();
  
  // Ports am Computer für die serielle Schnittstelle:
  String portnameUSB = Serial.list()[Serial.list().length-1];  // falls der Arduino über USB verbunden ist
  String portnameBluetooth = "/dev/rfcomm0";                   // falls der Arduino über Bluetooth verbunden ist
  serialObject = new Serial(this, portnameBluetooth, 9600);    // neues serielles Objekt
  serialObject.bufferUntil(10);                                // bis zum nächsten Zeilenumbruch (Charcode: 10) puffern
  printArray(Serial.list());
  vArduino = new VirtualArduino();                             
  seriCom = new SerialCommunication(serialObject, vArduino, servoDiagramVArduino, escDiagramVArduino, PTDiagramVArduino, distanceDiagramVArduino, batteryStatus);
  
  vController = new VirtualController();
  
  
  finUpDown = new ServoFin(seriCom, seriCom.FIN_UP_DOWN, 90, -63, 63); // ServoFin(SerialCommunication seriCom, int communikationKey, int nullPosition, int minPosition, int maxPosition)
  finLeftRight = new ServoFin(seriCom, seriCom.FIN_LEFT_RIGHT, 90, -45, 45);
  motorESC = new ServoFin(seriCom, seriCom.MOTOR_DRIVE, 0, 0, 180);
  
  ControlIO control = ControlIO.getInstance(this); // für GamePadAPI
  GamePadControl GPC = new GamePadControl(control, vController, finUpDown, finLeftRight, motorESC, servoDiagramVController, escDiagramVController);
  GPC.setup();                                     // für GamePadAPI; auskommentieren, wenn kein Game-Controller angeschlossen ist
  
  //finUpDown.setup(0);
  //finLeftRight.setup(0);
  //motorESC.setup(0);
 
  currentMode = MANUAL;                                        // aktueller Modus: manuelle Steuerung
  manualMode = new ManualMode(vController, GPC, vArduino, finUpDown, finLeftRight, motorESC, servoDiagramVController, escDiagramVController);
  lightMode = new LightFollowMode(vArduino, finUpDown, finLeftRight, motorESC, servoDiagramVController, escDiagramVController);
  
}

// die Hauptfunktion des aktuellen Modus wird aufgerufen:
void draw() {
  if (vArduino.getTimestamp() > 0) { // Wenn der vom Arduino gesendete Zeitstempel > 0 ist, also erst wenn bereits Daten vom Arduino gesendet wurden und das Setup am Arduino vorbei ist
    switch (currentMode) {
      case 0:
        break;
      case 1:
        manualMode.loop();
        break;
      case 2:
        lightMode.loop();
        break;
        }
  }
  
  // Wenn neue Daten von der seriellen Schnittstelle vorhanden sind, wird die entsprechende Methode in im seriCom-Objekt aufgerufen
  //println("Bytes vorhanden: " + str(serialObject.available())); // zum Debuggen
  if (serialObject.available() > 0) {
    seriCom.serialEvent();
  }
}

// Die Tasten 'c'=Kallibrierung,'m'=manuelle Steuerung und 'l'=Licht-folge-Modus bewirken den Wechsel des Modus.
// Im manuellen Modus lässt sich der Zeppelin mit 'w'=vorwärts, 'a'=links, 'd'=rechts, PFEILTASTE_OBEN=hoch, PFEILTASTE_UNTEN=runter steuern
void keyPressed() {
  switch (key) {
    case 'c':
      currentMode = CALLIBRATION;
      break;
    case 'm':
      currentMode = MANUAL;
      break;
    case 'l':
      currentMode = LIGHT;
      break;
    default:
      switch (currentMode) {
        case 0:
          break;
        case 1:                                 // Falls die Taste keinen Moduswechsel bewirkt und falls der aktuelle Modus manuelle Steuerung ist,
          manualMode.keyPressed(key, keyCode);  //  dann werden die entsprechenden Tasten-Codes an eine Methode des manualMode-Objekts übergeben und verarbeitet
          break;
        case 2:
          break;
      }
  }
}

void keyReleased() {
  switch (key) {
    case 'c':
      currentMode = CALLIBRATION;
      break;
    case 'm':
      currentMode = MANUAL;
      break;
    case 'l':
      currentMode = LIGHT;
      break;
    default:
      switch (currentMode) {
        case 0:
          break;
        case 1:
          manualMode.keyReleased(key, keyCode);
          break;
        case 2:
          break;
      }
  }
}
